# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from argcomplete.completers import FilesCompleter

from azure.cli.core.commands import register_cli_argument
from azure.cli.core.commands.parameters import (resource_group_name_type, location_type,
                                                get_resource_name_completion_list, file_type,
                                                CliArgumentType, ignore_type, enum_choice_list)
from azure.mgmt.web.models import DatabaseType, ConnectionStringType
from ._client_factory import web_client_factory
from ._validators import validate_existing_function_app, validate_existing_web_app


def _generic_site_operation(resource_group_name, name, operation_name, slot=None,
                            extra_parameter=None, client=None):
    client = client or web_client_factory()
    m = getattr(client.web_apps,
                operation_name if slot is None else operation_name + '_slot')
    if slot is None:
        return (m(resource_group_name, name)
                if extra_parameter is None else m(resource_group_name, name, extra_parameter))

    return (m(resource_group_name, name, slot)
            if extra_parameter is None else m(resource_group_name, name, extra_parameter, slot))


def get_hostname_completion_list(prefix, action, parsed_args, **kwargs):  # pylint: disable=unused-argument
    if parsed_args.resource_group_name and parsed_args.webapp:
        rg = parsed_args.resource_group_name
        webapp = parsed_args.webapp
        slot = getattr(parsed_args, 'slot', None)
        result = _generic_site_operation(rg, webapp, 'get_site_host_name_bindings', slot)
        # workaround an api defect, that 'name' is '<webapp>/<hostname>'
        return [r.name.split('/', 1)[1] for r in result]


# pylint: disable=line-too-long
# PARAMETER REGISTRATION
name_arg_type = CliArgumentType(options_list=('--name', '-n'), metavar='NAME')
sku_arg_type = CliArgumentType(help='The pricing tiers, e.g., F1(Free), D1(Shared), B1(Basic Small), B2(Basic Medium), B3(Basic Large), S1(Standard Small), P1(Premium Small), etc',
                               **enum_choice_list(['F1', 'FREE', 'D1', 'SHARED', 'B1', 'B2', 'B3', 'S1', 'S2', 'S3', 'P1', 'P2', 'P3']))
webapp_name_arg_type = CliArgumentType(configured_default='web', options_list=('--name', '-n'), metavar='NAME',
                                       completer=get_resource_name_completion_list('Microsoft.Web/sites'), id_part='name',
                                       help="name of the web. You can configure the default using 'az configure --defaults web=<name>'")

# use this hidden arg to give a command the right instance, that functionapp commands
# work on function app and webapp ones work on web app
register_cli_argument('webapp', 'app_instance', ignore_type)
register_cli_argument('functionapp', 'app_instance', ignore_type)
# function app doesn't have slot support
register_cli_argument('functionapp', 'slot', ignore_type)

register_cli_argument('appservice', 'resource_group_name', arg_type=resource_group_name_type)
register_cli_argument('appservice', 'location', arg_type=location_type)

register_cli_argument('appservice list-locations', 'linux_workers_enabled', action='store_true', help='get regions which support hosting webapps on Linux workers')
register_cli_argument('appservice list-locations', 'sku', arg_type=sku_arg_type)
register_cli_argument('appservice plan', 'name', arg_type=name_arg_type, help='The name of the app service plan', completer=get_resource_name_completion_list('Microsoft.Web/serverFarms'), id_part='name')
register_cli_argument('appservice plan create', 'name', options_list=('--name', '-n'), help="Name of the new app service plan", completer=None)
register_cli_argument('appservice plan create', 'sku', arg_type=sku_arg_type)
register_cli_argument('appservice plan create', 'is_linux', action='store_true', required=False, help='host webapp on Linux worker')
register_cli_argument('appservice plan update', 'sku', arg_type=sku_arg_type)
register_cli_argument('appservice plan update', 'allow_pending_state', ignore_type)
register_cli_argument('appservice plan', 'number_of_workers', help='Number of workers to be allocated.', type=int, default=1)
register_cli_argument('appservice plan', 'admin_site_name', help='The name of the admin web app.')

register_cli_argument('webapp', 'resource_group_name', arg_type=resource_group_name_type)
register_cli_argument('webapp', 'location', arg_type=location_type)

register_cli_argument('webapp', 'slot', options_list=('--slot', '-s'), help="the name of the slot. Default to the productions slot if not specified")
register_cli_argument('webapp', 'name', configured_default='web',
                      arg_type=name_arg_type, completer=get_resource_name_completion_list('Microsoft.Web/sites'), id_part='name',
                      help="name of the web. You can configure the default using 'az configure --defaults web=<name>'")
register_cli_argument('webapp create', 'name', options_list=('--name', '-n'), help='name of the new webapp')
register_cli_argument('webapp create', 'deployment_container_image_name', options_list=('--deployment-container-image-name', '-i'),
                      help='Linux only. Container image name from Docker Hub, e.g. publisher/image-name:tag')
register_cli_argument('webapp create', 'startup_file', help="Linux only. The web's startup file")
register_cli_argument('webapp create', 'runtime', options_list=('--runtime', '-r'), help="canonicalized web runtime in the format of Framework|Version, e.g. \"PHP|5.6\". Use 'az webapp list-runtimes' for available list")  # TODO ADD completer
register_cli_argument('webapp list-runtimes', 'linux', action='store_true', help='list runtime stacks for linux based webapps')  # TODO ADD completer
register_cli_argument('webapp traffic-routing', 'distribution', options_list=('--distribution', '-d'), nargs='+', help='space separated slot routings in a format of <slot-name>=<percentage> e.g. staging=50. Unused traffic percentage will go to the Production slot')

register_cli_argument('webapp create', 'plan', options_list=('--plan', '-p'), completer=get_resource_name_completion_list('Microsoft.Web/serverFarms'),
                      help="name or resource id of the app service plan. Use 'appservice plan create' to get one")

register_cli_argument('webapp browse', 'logs', options_list=('--logs', '-l'), action='store_true', help='Enable viewing the log stream immediately after launching the web app')
register_cli_argument('webapp delete', 'keep_empty_plan', action='store_true', help='keep empty app service plan')
register_cli_argument('webapp delete', 'keep_metrics', action='store_true', help='keep app metrics')
register_cli_argument('webapp delete', 'keep_dns_registration', action='store_true', help='keep DNS registration')

for scope in ['webapp', 'functionapp']:
    register_cli_argument(scope + ' config ssl bind', 'ssl_type', help='The ssl cert type', **enum_choice_list(['SNI', 'IP']))
    register_cli_argument(scope + ' config ssl upload', 'certificate_password', help='The ssl cert password')
    register_cli_argument(scope + ' config ssl upload', 'certificate_file', type=file_type, help='The filepath for the .pfx file')
    register_cli_argument(scope + ' config ssl', 'certificate_thumbprint', help='The ssl cert thumbprint')
    register_cli_argument(scope + ' config appsettings', 'settings', nargs='+', help="space separated app settings in a format of <name>=<value>")
    register_cli_argument(scope + ' config appsettings', 'setting_names', nargs='+', help="space separated app setting names")
    register_cli_argument(scope + ' config hostname', 'hostname', completer=get_hostname_completion_list, help="hostname assigned to the site, such as custom domains", id_part='child_name')
    register_cli_argument(scope + ' deployment user', 'user_name', help='user name')
    register_cli_argument(scope + ' deployment user', 'password', help='password, will prompt if not specified')
    register_cli_argument(scope + ' deployment source', 'manual_integration', action='store_true', help='disable automatic sync between source control and web')
    register_cli_argument(scope + ' deployment source', 'repo_url', options_list=('--repo-url', '-u'), help='repository url to pull the latest source from, e.g. https://github.com/foo/foo-web')
    register_cli_argument(scope + ' deployment source', 'branch', help='the branch name of the repository')
    register_cli_argument(scope + ' deployment source', 'cd_provider', help='type of CI/CD provider', default='kudu', **enum_choice_list(['kudu', 'vsts']))
    register_cli_argument(scope + ' deployment source', 'cd_app_type', arg_group='VSTS CD Provider', help='web application framework you used to develop your app', default='AspNetWap', **enum_choice_list(['AspNetWap', 'AspNetCore', 'NodeJSWithGulp', 'NodeJSWithGrunt']))
    register_cli_argument(scope + ' deployment source', 'cd_account', arg_group='VSTS CD Provider', help='name of the Team Services account to create/use for continuous delivery')
    register_cli_argument(scope + ' deployment source', 'cd_account_must_exist', arg_group='VSTS CD Provider', help='specifies that the account must already exist. If not specified, the account will be created if it does not already exist (existing accounts are updated)', action='store_true')
    register_cli_argument(scope + ' deployment source', 'repository_type', help='repository type', default='git', **enum_choice_list(['git', 'mercurial']))
    register_cli_argument(scope + ' deployment source', 'git_token', help='git access token required for auto sync')
    register_cli_argument(scope + ' create', 'deployment_local_git', action='store_true', options_list=('--deployment-local-git', '-l'), help='enable local git')
    register_cli_argument(scope + ' create', 'deployment_source_url', options_list=('--deployment-source-url', '-u'), help='Git repository URL to link with manual integration')
    register_cli_argument(scope + ' create', 'deployment_source_branch', options_list=('--deployment-source-branch', '-b'), help='the branch to deploy')

register_cli_argument('webapp config hostname', 'webapp_name', help="webapp name. You can configure the default using 'az configure --defaults web=<name>'", configured_default='web',
                      completer=get_resource_name_completion_list('Microsoft.Web/sites'), id_part='name')
register_cli_argument('webapp config appsettings', 'slot_settings', nargs='+', help="space separated slot app settings in a format of <name>=<value>")

two_states_switch = ['true', 'false']

register_cli_argument('webapp deployment container config', 'enable', options_list=('--enable-cd', '-e'), help='enable/disable continuous deployment', **enum_choice_list(two_states_switch))
register_cli_argument('webapp deployment slot', 'slot', help='the name of the slot')
register_cli_argument('webapp deployment slot', 'webapp', arg_type=name_arg_type, completer=get_resource_name_completion_list('Microsoft.Web/sites'),
                      help='Name of the webapp', id_part='name')
register_cli_argument('webapp deployment slot', 'auto_swap_slot', help='target slot to auto swap', default='production')
register_cli_argument('webapp deployment slot', 'disable', help='disable auto swap', action='store_true')
register_cli_argument('webapp deployment slot', 'target_slot', help="target slot to swap, default to 'production'")
register_cli_argument('webapp deployment slot create', 'configuration_source', help="source slot to clone configurations from. Use webapp's name to refer to the production slot")
register_cli_argument('webapp deployment slot swap', 'action', help="swap types. use 'preview' to apply target slot's settings on the source slot first; use 'swap' to complete it; use 'reset' to reset the swap",
                      **enum_choice_list(['swap', 'preview', 'reset']))

register_cli_argument('webapp log config', 'application_logging', help='configure application logging to file system', **enum_choice_list(two_states_switch))
register_cli_argument('webapp log config', 'detailed_error_messages', help='configure detailed error messages', **enum_choice_list(two_states_switch))
register_cli_argument('webapp log config', 'failed_request_tracing', help='configure failed request tracing', **enum_choice_list(two_states_switch))
register_cli_argument('webapp log config', 'level', help='logging level', **enum_choice_list(['error', 'warning', 'information', 'verbose']))
server_log_switch_options = ['off', 'storage', 'filesystem']
register_cli_argument('webapp log config', 'web_server_logging', help='configure Web server logging', **enum_choice_list(server_log_switch_options))

register_cli_argument('webapp log tail', 'provider', help="By default all live traces configured by 'az webapp log config' will be shown, but you can scope to certain providers/folders, e.g. 'application', 'http', etc. For details, check out https://github.com/projectkudu/kudu/wiki/Diagnostic-Log-Stream")
register_cli_argument('webapp log download', 'log_file', default='webapp_logs.zip', type=file_type, completer=FilesCompleter(), help='the downloaded zipped log file path')

for scope in ['appsettings', 'connection-string']:
    register_cli_argument('webapp config ' + scope, 'settings', nargs='+', help="space separated {} in a format of <name>=<value>".format(scope))
    register_cli_argument('webapp config ' + scope, 'slot_settings', nargs='+', help="space separated slot {} in a format of <name>=<value>".format(scope))
    register_cli_argument('webapp config ' + scope, 'setting_names', nargs='+', help="space separated {} names".format(scope))
register_cli_argument('webapp config connection-string', 'connection_string_type',
                      options_list=('--connection-string-type', '-t'), help='connection string type', **enum_choice_list(ConnectionStringType))

register_cli_argument('webapp config container', 'docker_registry_server_url', options_list=('--docker-registry-server-url', '-r'), help='the container registry server url')
register_cli_argument('webapp config container', 'docker_custom_image_name', options_list=('--docker-custom-image-name', '-c', '-i'), help='the container custom image name and optionally the tag name')
register_cli_argument('webapp config container', 'docker_registry_server_user', options_list=('--docker-registry-server-user', '-u'), help='the container registry server username')
register_cli_argument('webapp config container', 'docker_registry_server_password', options_list=('--docker-registry-server-password', '-p'), help='the container registry server password')

register_cli_argument('webapp config set', 'remote_debugging_enabled', help='enable or disable remote debugging', **enum_choice_list(two_states_switch))
register_cli_argument('webapp config set', 'web_sockets_enabled', help='enable or disable web sockets', **enum_choice_list(two_states_switch))
register_cli_argument('webapp config set', 'always_on', help='ensure webapp gets loaded all the time, rather unloaded after been idle. Recommended when you have continuous web jobs running', **enum_choice_list(two_states_switch))
register_cli_argument('webapp config set', 'auto_heal_enabled', help='enable or disable auto heal', **enum_choice_list(two_states_switch))
register_cli_argument('webapp config set', 'use32_bit_worker_process', options_list=('--use-32bit-worker-process',), help='use 32 bits worker process or not', **enum_choice_list(two_states_switch))
register_cli_argument('webapp config set', 'php_version', help='The version used to run your web app if using PHP, e.g., 5.5, 5.6, 7.0')
register_cli_argument('webapp config set', 'python_version', help='The version used to run your web app if using Python, e.g., 2.7, 3.4')
register_cli_argument('webapp config set', 'net_framework_version', help="The version used to run your web app if using .NET Framework, e.g., 'v4.0' for .NET 4.6 and 'v3.0' for .NET 3.5")
register_cli_argument('webapp config set', 'linux_fx_version', help="The runtime stack used for your linux-based webapp, e.g., \"RUBY|2.3\", \"NODE|6.6\", \"PHP|5.6\", \"DOTNETCORE|1.1.0\". See https://aka.ms/linux-stacks for more info.")
register_cli_argument('webapp config set', 'java_version', help="The version used to run your web app if using Java, e.g., '1.7' for Java 7, '1.8' for Java 8")
register_cli_argument('webapp config set', 'java_container', help="The java container, e.g., Tomcat, Jetty")
register_cli_argument('webapp config set', 'java_container_version', help="The version of the java container, e.g., '8.0.23' for Tomcat")
register_cli_argument('webapp config set', 'app_command_line', options_list=('--startup-file',), help="The startup file for linux hosted web apps, e.g. 'process.json' for Node.js web")

register_cli_argument('webapp config backup', 'storage_account_url', help='URL with SAS token to the blob storage container', options_list=['--container-url'])
register_cli_argument('webapp config backup', 'webapp_name', help='The name of the webapp')
register_cli_argument('webapp config backup', 'db_name', help='Name of the database in the backup', arg_group='Database')
register_cli_argument('webapp config backup', 'db_connection_string', help='Connection string for the database in the backup', arg_group='Database')
register_cli_argument('webapp config backup', 'db_type', help='Type of database in the backup', arg_group='Database', **enum_choice_list(DatabaseType))

register_cli_argument('webapp config backup create', 'backup_name', help='Name of the backup. If unspecified, the backup will be named with the webapp name and a timestamp')

register_cli_argument('webapp config backup update', 'frequency', help='How often to backup. Use a number followed by d or h, e.g. 5d = 5 days, 2h = 2 hours')
register_cli_argument('webapp config backup update', 'keep_at_least_one_backup', help='Always keep one backup, regardless of how old it is', options_list=['--retain-one'], **enum_choice_list(two_states_switch))
register_cli_argument('webapp config backup update', 'retention_period_in_days', help='How many days to keep a backup before automatically deleting it. Set to 0 for indefinite retention', options_list=['--retention'])

register_cli_argument('webapp config backup restore', 'backup_name', help='Name of the backup to restore')
register_cli_argument('webapp config backup restore', 'target_name', help='The name to use for the restored webapp. If unspecified, will default to the name that was used when the backup was created')
register_cli_argument('webapp config backup restore', 'overwrite', help='Overwrite the source webapp, if --target-name is not specified', action='store_true')
register_cli_argument('webapp config backup restore', 'ignore_hostname_conflict', help='Ignores custom hostnames stored in the backup', action='store_true')

register_cli_argument('functionapp', 'name', arg_type=name_arg_type, id_part='name', help='name of the function app')
register_cli_argument('functionapp config hostname', 'webapp_name', arg_type=name_arg_type, id_part='name', help='name of the function app')
register_cli_argument('functionapp create', 'plan', options_list=('--plan', '-p'), completer=get_resource_name_completion_list('Microsoft.Web/serverFarms'),
                      help="name or resource id of the function app service plan. Use 'appservice plan create' to get one")
register_cli_argument('functionapp create', 'new_app_name', options_list=('--name', '-n'), help='name of the new function app')
register_cli_argument('functionapp create', 'storage_account', options_list=('--storage-account', '-s'),
                      help='Provide a string value of a Storage Account in the provided Resource Group. Or Resource ID of a Storage Account in a different Resource Group')

# For commands with shared impl between webapp and functionapp and has output, we apply type validation to avoid confusions
register_cli_argument('appservice web show', 'name', arg_type=webapp_name_arg_type, validator=validate_existing_web_app)
register_cli_argument('webapp show', 'name', arg_type=webapp_name_arg_type, validator=validate_existing_web_app)
register_cli_argument('functionapp show', 'name', arg_type=name_arg_type, validator=validate_existing_function_app)

register_cli_argument('functionapp', 'name', arg_type=name_arg_type, id_part='name', help="name of the function")
register_cli_argument('functionapp create', 'plan', options_list=('--plan', '-p'), completer=get_resource_name_completion_list('Microsoft.Web/serverFarms'),
                      help="name or resource id of the function app service plan. Use 'appservice plan create' to get one")
register_cli_argument('functionapp create', 'consumption_plan_location', options_list=('--consumption-plan-location', '-c'),
                      help="Geographic location where Function App will be hosted. Use 'functionapp list-consumption-locations' to view available locations.")
register_cli_argument('functionapp create', 'storage_account', options_list=('--storage-account', '-s'),
                      help='Provide a string value of a Storage Account in the provided Resource Group. Or Resource ID of a Storage Account in a different Resource Group')



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

# pylint: disable=line-too-long
from azure.cli.core.application import APPLICATION
from azure.cli.core.commands import cli_command
from azure.cli.core.commands.arm import cli_generic_update_command
from azure.cli.core.util import empty_on_404
from azure.cli.core.profiles import supported_api_version, PROFILE_TYPE

from ._client_factory import cf_web_client, cf_plans


def deprecate(argv):
    if len(argv) > 1 and argv[0] == 'appservice' and argv[1] == 'web':
        from azure.cli.core.util import CLIError
        raise CLIError("All 'appservice web' commands have been renamed to 'webapp'")


APPLICATION.register(APPLICATION.COMMAND_PARSER_PARSING, deprecate)


def output_slots_in_table(slots):
    return [{'name': s['name'], 'status': s['state'], 'plan': s['appServicePlan']} for s in slots]


def transform_list_location_output(result):
    return [{'name': x.name} for x in result]


def transform_web_output(web):
    props = ['name', 'state', 'location', 'resourceGroup', 'defaultHostName', 'appServicePlanId', 'ftpPublishingUrl']
    result = {k: web[k] for k in web if k in props}
    # to get width under control, also the plan usually is in the same RG
    result['appServicePlan'] = result.pop('appServicePlanId').split('/')[-1]
    return result


def transform_web_list_output(webs):
    return [transform_web_output(w) for w in webs]


def ex_handler_factory(creating_plan=False):
    def _polish_bad_errors(ex):
        import json
        from azure.cli.core.util import CLIError
        try:
            detail = json.loads(ex.response.text)['Message']
            if creating_plan:
                if 'Requested features are not supported in region' in detail:
                    detail = ("Plan with linux worker is not supported in current region. For " +
                              "supported regions, please refer to https://docs.microsoft.com/en-us/"
                              "azure/app-service-web/app-service-linux-intro")
                elif 'Not enough available reserved instance servers to satisfy' in detail:
                    detail = ("Plan with Linux worker can only be created in a group " +
                              "which has never contained a Windows worker, and vice versa. " +
                              "Please use a new resource group. Original error:" + detail)
            ex = CLIError(detail)
        except Exception:  # pylint: disable=broad-except
            pass
        raise ex
    return _polish_bad_errors


custom_path = 'azure.cli.command_modules.appservice.custom#'

cli_command(__name__, 'webapp create', custom_path + 'create_webapp', exception_handler=ex_handler_factory())
cli_command(__name__, 'webapp list', custom_path + 'list_webapp', table_transformer=transform_web_list_output)
cli_command(__name__, 'webapp show', custom_path + 'show_webapp', exception_handler=empty_on_404, table_transformer=transform_web_output)
cli_command(__name__, 'webapp delete', custom_path + 'delete_webapp')
cli_command(__name__, 'webapp stop', custom_path + 'stop_webapp')
cli_command(__name__, 'webapp start', custom_path + 'start_webapp')
cli_command(__name__, 'webapp restart', custom_path + 'restart_webapp')
cli_command(__name__, 'webapp traffic-routing set', custom_path + 'set_traffic_routing')
cli_command(__name__, 'webapp traffic-routing show', custom_path + 'show_traffic_routing')
cli_command(__name__, 'webapp traffic-routing clear', custom_path + 'clear_traffic_routing')

cli_command(__name__, 'webapp config set', custom_path + 'update_site_configs')
cli_command(__name__, 'webapp config show', custom_path + 'get_site_configs', exception_handler=empty_on_404)
cli_command(__name__, 'webapp config appsettings list', custom_path + 'get_app_settings', exception_handler=empty_on_404)
cli_command(__name__, 'webapp config appsettings set', custom_path + 'update_app_settings')
cli_command(__name__, 'webapp config appsettings delete', custom_path + 'delete_app_settings')
cli_command(__name__, 'webapp config connection-string list', custom_path + 'get_connection_strings', exception_handler=empty_on_404)
cli_command(__name__, 'webapp config connection-string set', custom_path + 'update_connection_strings')
cli_command(__name__, 'webapp config connection-string delete', custom_path + 'delete_connection_strings')

cli_command(__name__, 'webapp config hostname add', custom_path + 'add_hostname', exception_handler=ex_handler_factory())
cli_command(__name__, 'webapp config hostname list', custom_path + 'list_hostnames')
cli_command(__name__, 'webapp config hostname delete', custom_path + 'delete_hostname')
cli_command(__name__, 'webapp config hostname get-external-ip', custom_path + 'get_external_ip')
cli_command(__name__, 'webapp config container set', custom_path + 'update_container_settings')
cli_command(__name__, 'webapp config container delete', custom_path + 'delete_container_settings')
cli_command(__name__, 'webapp config container show', custom_path + 'show_container_settings', exception_handler=empty_on_404)

cli_command(__name__, 'webapp config ssl upload', custom_path + 'upload_ssl_cert', exception_handler=ex_handler_factory())
cli_command(__name__, 'webapp config ssl list', custom_path + 'list_ssl_certs')
cli_command(__name__, 'webapp config ssl bind', custom_path + 'bind_ssl_cert', exception_handler=ex_handler_factory())
cli_command(__name__, 'webapp config ssl unbind', custom_path + 'unbind_ssl_cert')
cli_command(__name__, 'webapp config ssl delete', custom_path + 'delete_ssl_cert')

cli_command(__name__, 'webapp config backup list', custom_path + 'list_backups')
cli_command(__name__, 'webapp config backup show', custom_path + 'show_backup_configuration', exception_handler=empty_on_404)
cli_command(__name__, 'webapp config backup create', custom_path + 'create_backup', exception_handler=ex_handler_factory())
cli_command(__name__, 'webapp config backup update', custom_path + 'update_backup_schedule', exception_handler=ex_handler_factory())
cli_command(__name__, 'webapp config backup restore', custom_path + 'restore_backup', exception_handler=ex_handler_factory())

cli_command(__name__, 'webapp deployment source config-local-git', custom_path + 'enable_local_git')
cli_command(__name__, 'webapp deployment source config', custom_path + 'config_source_control', exception_handler=ex_handler_factory())
cli_command(__name__, 'webapp deployment source sync', custom_path + 'sync_site_repo', exception_handler=ex_handler_factory())
cli_command(__name__, 'webapp deployment source show', custom_path + 'show_source_control', exception_handler=empty_on_404)
cli_command(__name__, 'webapp deployment source delete', custom_path + 'delete_source_control')
cli_command(__name__, 'webapp deployment source update-token', custom_path + 'update_git_token', exception_handler=ex_handler_factory())

cli_command(__name__, 'webapp log tail', custom_path + 'get_streaming_log')
cli_command(__name__, 'webapp log download', custom_path + 'download_historical_logs')
cli_command(__name__, 'webapp log config', custom_path + 'config_diagnostics')
cli_command(__name__, 'webapp log show', custom_path + 'show_diagnostic_settings')
cli_command(__name__, 'webapp browse', custom_path + 'view_in_browser')

cli_command(__name__, 'webapp deployment slot list', custom_path + 'list_slots', table_transformer=output_slots_in_table)
cli_command(__name__, 'webapp deployment slot delete', custom_path + 'delete_slot')
cli_command(__name__, 'webapp deployment slot auto-swap', custom_path + 'config_slot_auto_swap')
cli_command(__name__, 'webapp deployment slot swap', custom_path + 'swap_slot', exception_handler=ex_handler_factory())
cli_command(__name__, 'webapp deployment slot create', custom_path + 'create_webapp_slot', exception_handler=ex_handler_factory())

cli_command(__name__, 'webapp deployment user set', custom_path + 'set_deployment_user')
cli_command(__name__, 'webapp deployment list-publishing-profiles',
            custom_path + 'list_publish_profiles')
cli_command(__name__, 'webapp deployment container config',
            custom_path + 'enable_cd')
cli_command(__name__, 'webapp deployment container show-cd-url',
            custom_path + 'show_container_cd_url')
cli_command(__name__, 'webapp deployment user show', 'azure.mgmt.web.web_site_management_client#WebSiteManagementClient.get_publishing_user', cf_web_client, exception_handler=empty_on_404)
cli_command(__name__, 'webapp list-runtimes', custom_path + 'list_runtimes')

if not supported_api_version(PROFILE_TYPE, max_api='2017-03-09-profile'):
    cli_command(__name__, 'appservice plan create', custom_path + 'create_app_service_plan', exception_handler=ex_handler_factory(creating_plan=True))
    cli_command(__name__, 'appservice plan delete', 'azure.mgmt.web.operations.app_service_plans_operations#AppServicePlansOperations.delete', cf_plans, confirmation=True)
    cli_command(__name__, 'appservice plan list', custom_path + 'list_app_service_plans')
    cli_command(__name__, 'appservice plan show', 'azure.mgmt.web.operations.app_service_plans_operations#AppServicePlansOperations.get', cf_plans, exception_handler=empty_on_404)
    cli_generic_update_command(__name__, 'appservice plan update', 'azure.mgmt.web.operations.app_service_plans_operations#AppServicePlansOperations.get',
                               'azure.mgmt.web.operations.app_service_plans_operations#AppServicePlansOperations.create_or_update',
                               custom_function_op=custom_path + 'update_app_service_plan',
                               setter_arg_name='app_service_plan', factory=cf_plans)
    cli_command(__name__, 'appservice list-locations', 'azure.mgmt.web.web_site_management_client#WebSiteManagementClient.list_geo_regions', cf_web_client, transform=transform_list_location_output)

    cli_command(__name__, 'functionapp create', custom_path + 'create_function')
    cli_command(__name__, 'functionapp list', custom_path + 'list_function_app', table_transformer=transform_web_list_output)
    cli_command(__name__, 'functionapp show', custom_path + 'show_webapp', exception_handler=empty_on_404, table_transformer=transform_web_output)
    cli_command(__name__, 'functionapp delete', custom_path + 'delete_function_app')
    cli_command(__name__, 'functionapp stop', custom_path + 'stop_webapp')
    cli_command(__name__, 'functionapp start', custom_path + 'start_webapp')
    cli_command(__name__, 'functionapp restart', custom_path + 'restart_webapp')
    cli_command(__name__, 'functionapp list-consumption-locations', custom_path + 'list_consumption_locations')
    cli_command(__name__, 'functionapp config appsettings list', custom_path + 'get_app_settings', exception_handler=empty_on_404)
    cli_command(__name__, 'functionapp config appsettings set', custom_path + 'update_app_settings')
    cli_command(__name__, 'functionapp config appsettings delete', custom_path + 'delete_app_settings')
    cli_command(__name__, 'functionapp config hostname add', custom_path + 'add_hostname', exception_handler=ex_handler_factory())
    cli_command(__name__, 'functionapp config hostname list', custom_path + 'list_hostnames')
    cli_command(__name__, 'functionapp config hostname delete', custom_path + 'delete_hostname')
    cli_command(__name__, 'functionapp config hostname get-external-ip', custom_path + 'get_external_ip')
    cli_command(__name__, 'functionapp config ssl upload', custom_path + 'upload_ssl_cert', exception_handler=ex_handler_factory())
    cli_command(__name__, 'functionapp config ssl list', custom_path + 'list_ssl_certs')
    cli_command(__name__, 'functionapp config ssl bind', custom_path + 'bind_ssl_cert', exception_handler=ex_handler_factory())
    cli_command(__name__, 'functionapp config ssl unbind', custom_path + 'unbind_ssl_cert')
    cli_command(__name__, 'functionapp config ssl delete', custom_path + 'delete_ssl_cert')
    cli_command(__name__, 'functionapp deployment source config-local-git', custom_path + 'enable_local_git')
    cli_command(__name__, 'functionapp deployment source config', custom_path + 'config_source_control', exception_handler=ex_handler_factory())
    cli_command(__name__, 'functionapp deployment source sync', custom_path + 'sync_site_repo')
    cli_command(__name__, 'functionapp deployment source show', custom_path + 'show_source_control', exception_handler=empty_on_404)
    cli_command(__name__, 'functionapp deployment source delete', custom_path + 'delete_source_control')
    cli_command(__name__, 'functionapp deployment source update-token', custom_path + 'update_git_token', exception_handler=ex_handler_factory())
    cli_command(__name__, 'functionapp deployment user set', custom_path + 'set_deployment_user')
    cli_command(__name__, 'functionapp deployment list-publishing-profiles',
                custom_path + 'list_publish_profiles')
    cli_command(__name__, 'functionapp deployment user show', 'azure.mgmt.web.web_site_management_client#WebSiteManagementClient.get_publishing_user', cf_web_client, exception_handler=empty_on_404)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure.cli.core.util import CLIError

from ._client_factory import web_client_factory


def validate_existing_function_app(namespace):
    validate_existing_app(namespace, 'functionapp')


def validate_existing_web_app(namespace):
    validate_existing_app(namespace, 'app')


def validate_existing_app(namespace, app_type):
    client = web_client_factory()
    instance = client.web_apps.get(namespace.resource_group_name, namespace.name)
    if instance.kind != app_type:  # pylint: disable=no-member
        raise CLIError('Usage Error: {} is not a correct app type'.format(namespace.name))
    setattr(namespace, 'app_instance', instance)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure.cli.core.help_files import helps


helps['appservice'] = """
    type: group
    short-summary: Manage your App Service plans.
"""

helps['webapp'] = """
    type: group
    short-summary: Manage web apps.
"""

helps['webapp config'] = """
    type: group
    short-summary: Configure a web app.
"""

helps['webapp config show'] = """
    type: command
    short-summary: Show web app configurations.
"""

helps['webapp config set'] = """
    type: command
    short-summary: create or update web app configurations.
"""

helps['webapp config appsettings'] = """
    type: group
    short-summary: Configure web app settings.
"""

helps['webapp config appsettings delete'] = """
    type: command
    short-summary: Delete web app settings.
"""

helps['webapp config appsettings list'] = """
    type: command
    short-summary: Show web app settings.
"""

helps['webapp config appsettings set'] = """
    type: command
    short-summary: Create or update web app settings.
    examples:
        - name: Set the default node version for a specified web app.
          text: >
            az webapp config appsettings set
            -g MyResourceGroup
            -n MyUniqueApp
            --settings WEBSITE_NODE_DEFAULT_VERSION=6.9.1
"""

helps['webapp config connection-string'] = """
    type: group
    short-summary: Configure web app connection strings.
"""

helps['webapp config connection-string show'] = """
    type: command
    short-summary: Show connection strings
"""

helps['webapp config connection-string delete'] = """
    type: command
    short-summary: delete connection strings
"""

helps['webapp config connection-string set'] = """
    type: command
    short-summary: Create or update connection strings.
    examples:
        - name: add a mysql connection string.
          text: >
            az webapp config connection-string set
            -g MyResourceGroup
            -n MyUniqueApp
            -t mysql
            --settings mysql1='Server=myServer;Database=myDB;Uid=myUser;Pwd=myPwd;'
"""

helps['webapp config container'] = """
    type: group
    short-summary: Configure container specific settings.
"""

helps['webapp config container show'] = """
    type: command
    short-summary: Show container settings.
"""

helps['webapp config container set'] = """
    type: command
    short-summary: create or update container settings.
"""

helps['webapp config container delete'] = """
    type: command
    short-summary: Delete container settings.
"""

helps['webapp config hostname'] = """
    type: group
    short-summary: Configure hostnames.
"""

helps['webapp config ssl'] = """
    type: group
    short-summary: Configure SSL certificates.
"""

helps['webapp config ssl list'] = """
    type: command
    short-summary: List SSL certificates within a resource group
"""

helps['webapp config ssl bind'] = """
    type: command
    short-summary: Bind an SSL certificate to a web app.
"""

helps['webapp config ssl unbind'] = """
    type: command
    short-summary: Unbind an SSL certificate from a web app.
"""

helps['webapp config ssl delete'] = """
    type: command
    short-summary: Delete an SSL certificate from a web app.
"""

helps['webapp config ssl upload'] = """
    type: command
    short-summary: Upload an SSL certificate to a web app.
"""

helps['webapp deployment'] = """
    type: group
    short-summary: Manage web app deployments.
"""

helps['webapp deployment source'] = """
    type: group
    short-summary: Manage deployment source repositories.
"""


helps['webapp deployment slot'] = """
    type: group
    short-summary: Manage web app deployment slots.
"""

helps['webapp deployment slot auto-swap'] = """
    type: group
    short-summary: Enable or disable auto-swap for a web app deployment slot.
"""

helps['webapp log'] = """
    type: group
    short-summary: Manage web app logs.
"""

helps['webapp log config'] = """
    type: command
    short-summary: Configure web app logs.
"""

helps['webapp log show'] = """
    type: command
    short-summary: Show web app log configurations.
"""

helps['webapp log download'] = """
    type: command
    short-summary: Download historical logs as a zip file
    long-summary: Might not work with Linux webs
"""

helps['webapp log tail'] = """
    type: command
    short-summary: Start live tracing
    long-summary: Might not work with Linux webs
"""

helps['webapp deployment'] = """
    type: group
    short-summary: Manage web application deployments.
"""

helps['webapp deployment list-publishing-profiles'] = """
    type: command
    short-summary: get publishing endpoints, credentials, database connection strings, etc
"""

helps['webapp deployment container'] = """
    type: group
    short-summary: Manage container based continuous deployment.
"""

helps['webapp deployment container config'] = """
    type: command
    short-summary: configure continuous deployment.
"""

helps['webapp deployment container show-cd-url'] = """
    type: command
    short-summary: get the webhook url of the linux webapp you can use to configure the container repository for continuous deployment
"""

helps['webapp deployment slot auto-swap'] = """
    type: command
    short-summary: Configure slot auto swap.
"""

helps['webapp deployment slot create'] = """
    type: command
    short-summary: Create a slot.
"""

helps['webapp deployment slot swap'] = """
    type: command
    short-summary: Swap slots.
    examples:
        - name: Swap a staging slot into production for the specified web app.
          text: >
            az webapp deployment slot swap
            -g MyResourceGroup
            -n MyUniqueApp
            --slot staging
            --target-slot production
"""

helps['webapp deployment slot list'] = """
    type: command
    short-summary: List all slots.
"""

helps['webapp deployment slot delete'] = """
    type: command
    short-summary: Delete a slot.
"""

helps['webapp deployment user'] = """
    type: group
    short-summary: Manage user credentials for a deployment.
"""

helps['webapp deployment user set'] = """
    type: command
    short-summary: Update deployment credentials.
    long-summary: All web apps in the subscription will be impacted since all web apps share
                  the same deployment credentials.
    examples:
        - name: Set FTP and git deployment credentials for all web apps.
          text: >
            az webapp deployment user set
            --user-name MyUserName
"""

helps['webapp deployment slot'] = """
    type: group
    short-summary: Manage deployment slots.
"""

helps['webapp deployment source'] = """
    type: group
    short-summary: Manage source control systems.
"""

helps['webapp deployment source config'] = """
    type: command
    short-summary: Associate to Git or Mercurial repositories.
"""

helps['webapp deployment source config-local-git'] = """
    type: command
    short-summary: Enable local git.
    long-summary: Get an endpoint to clone and later push to the web app.
    examples:
        - name: Get a git endpoint for a web app and add it as a remote.
          text: >
            az webapp source-control config-local-git \\
                -g MyResourceGroup -n MyUniqueApp

            git remote add azure \\
                https://<deploy_user_name>@MyUniqueApp.scm.azurewebsites.net/MyUniqueApp.git
"""

helps['webapp deployment source delete'] = """
    type: command
    short-summary: Delete source control configurations.
"""

helps['webapp deployment source show'] = """
    type: command
    short-summary: Show source control configurations.
"""

helps['webapp deployment source sync'] = """
    type: command
    short-summary: Synchronize from the source repository, only needed under manual integration mode.
"""

helps['webapp traffic-routing'] = """
    type: group
    short-summary: Manage traffic routings in production test.
"""

helps['webapp traffic-routing'] = """
    type: group
    short-summary: Manage traffic routings in production test.
"""

helps['webapp traffic-routing set'] = """
    type: command
    short-summary: Routing some percentages of traffic to deployment slots
"""

helps['webapp traffic-routing show'] = """
    type: command
    short-summary: Display the current distribution of traffic across slots
"""

helps['webapp traffic-routing clear'] = """
    type: command
    short-summary: Clear the routing rules to send 100% to production
"""

helps['appservice plan'] = """
    type: group
    short-summary: Manage App Service plans.
"""

helps['appservice plan update'] = """
    type: command
    short-summary: Update an App Service plan.
"""

helps['appservice plan create'] = """
    type: command
    short-summary: Create an App Service plan.
    examples:
        - name: Create a basic App Service plan.
          text: >
            az appservice plan create -g MyResourceGroup -n MyPlan
        - name: Create a standard App Service plan with with four Linux workers.
          text: >
            az appservice plan create
            -g MyResourceGroup
            -n MyPlan
            --is-linux
            --number-of-workers 4
            --sku S1
"""

helps['appservice plan delete'] = """
    type: command
    short-summary: Delete an App Service plan.
"""

helps['appservice plan list'] = """
    type: command
    short-summary: List App Service plans.
    examples:
        - name: List all free tier App Service plans.
          text: >
            az appservice plan list --query "[?sku.tier=='Free']"
"""

helps['appservice plan show'] = """
    type: command
    short-summary: Get the App Service plans for a resource group or a set of resource groups.
"""

helps['webapp config hostname add'] = """
    type: command
    short-summary: Bind a hostname (custom domain) to a web app.
"""

helps['webapp config hostname delete'] = """
    type: command
    short-summary: Unbind a hostname (custom domain) from a web app.
"""

helps['webapp config hostname list'] = """
    type: command
    short-summary: List all hostname bindings.
"""

helps['webapp config hostname get-external-ip'] = """
    type: command
    short-summary: get the ip address to configure your DNS settings for A records
"""

helps['webapp config backup list'] = """
    type: command
    short-summary: List all backups of a web app.
"""

helps['webapp config backup create'] = """
    type: command
    short-summary: Create a backup of a web app.
"""

helps['webapp config backup show'] = """
    type: command
    short-summary: Show the backup schedule of a web app.
"""

helps['webapp config backup update'] = """
    type: command
    short-summary: Configure a new backup schedule.
"""

helps['webapp config backup restore'] = """
    type: command
    short-summary: Restore a web app from a backup.
"""

helps['webapp browse'] = """
    type: command
    short-summary: Open the web app in a browser.
"""

helps['webapp create'] = """
    type: command
    short-summary: Create a web app.
    examples:
        - name: Create an empty webapp.  Name must be unique to yield a unique FQDN;
                for example, MyUniqueApp.azurewebsites.net.
          text: >
            az webapp create -g MyResourceGroup -p MyPlan -n MyUniqueApp
        - name: Create a webapp with node 6.2 stack runtime, and local git configured for web deployment
          text: >
            az webapp create -g MyResourceGroup -p MyPlan -n MyUniqueApp --runtime "node|6.2" --deployment-local-git
"""

helps['webapp list-runtimes'] = """
    type: command
    short-summary: List built-in web stack runtimes you can use to create new webapps.
"""

helps['webapp delete'] = """
    type: command
    short-summary: Delete a web app.
"""

helps['webapp list'] = """
    type: command
    short-summary: List web apps.
    examples:
        - name: List default host name and state for all web apps.
          text: >
            az webapp list --query "[].{ hostName: defaultHostName, state: state }"
        - name: List all running web apps.
          text: >
            az webapp list --query "[?state=='Running']"
"""

helps['webapp restart'] = """
    type: command
    short-summary: Restart a web app.
"""

helps['webapp start'] = """
    type: command
    short-summary: Start a web app.
"""

helps['webapp show'] = """
    type: command
    short-summary: Show a web app.
"""

helps['webapp stop'] = """
    type: command
    short-summary: Stop a web app.
"""

helps['webapp production-test'] = """
    type: command
    short-summary: test in production, including configuring static routings.
"""

helps['functionapp'] = """
    type: group
    short-summary: Manage your function app.
"""

helps['functionapp create'] = """
    type: command
    short-summary: Create a function app.
    examples:
        - name: Create a basic function app.  Name must be unique to yield a unique FQDN;
                for example, MyUniqueApp.azurewebsites.net.
          text: >
            az functionapp create
            -g MyResourceGroup
            -p MyPlan
            -n MyUniqueApp
            -s MyStorageAccount
"""

helps['functionapp delete'] = """
    type: command
    short-summary: Delete a function app.
"""

helps['functionapp list'] = """
    type: command
    short-summary: List function apps.
    examples:
        - name: List default host name and state for all function apps.
          text: >
            az functionapp list --query "[].{ hostName: defaultHostName, state: state }"
        - name: List all running function apps.
          text: >
            az functionapp list --query "[?state=='Running']"
"""

helps['functionapp restart'] = """
    type: command
    short-summary: Restart a function app.
"""

helps['functionapp start'] = """
    type: command
    short-summary: Start a function app.
"""

helps['functionapp show'] = """
    type: command
    short-summary: Show a function app.
"""

helps['functionapp stop'] = """
    type: command
    short-summary: Stop a function app.
"""

helps['functionapp config'] = """
    type: group
    short-summary: Configure a function app.
"""

helps['functionapp config appsettings'] = """
    type: group
    short-summary: Configure function app settings.
"""

helps['functionapp config appsettings show'] = """
    type: command
    short-summary: Show function app settings.
"""

helps['functionapp config appsettings set'] = """
    type: command
    short-summary: Create or update function app settings.
"""

helps['functionapp config hostname'] = """
    type: group
    short-summary: Configure hostnames.
"""
helps['functionapp config hostname add'] = """
    type: command
    short-summary: Bind a hostname (custom domain) to a function app.
"""

helps['functionapp config hostname delete'] = """
    type: command
    short-summary: Unbind a hostname (custom domain) from a function app.
"""

helps['functionapp config hostname list'] = """
    type: command
    short-summary: List all hostname bindings.
"""

helps['functionapp config hostname get-external-ip'] = """
    type: command
    short-summary: get the ip address to configure your DNS settings for A records
"""

helps['functionapp config ssl'] = """
    type: group
    short-summary: Configure SSL certificates.
"""

helps['functionapp config ssl list'] = """
    type: command
    short-summary: List SSL certificates within a resource group
"""

helps['functionapp config ssl bind'] = """
    type: command
    short-summary: Bind an SSL certificate to a function app.
"""

helps['functionapp config ssl unbind'] = """
    type: command
    short-summary: Unbind an SSL certificate from a function app.
"""

helps['functionapp config ssl delete'] = """
    type: command
    short-summary: Delete an SSL certificate from a function app.
"""

helps['functionapp config ssl upload'] = """
    type: command
    short-summary: Upload an SSL certificate to a function app.
"""
helps['functionapp deployment'] = """
    type: group
    short-summary: Manage function app deployments.
"""
helps['functionapp deployment source'] = """
    type: group
    short-summary: Manage source control systems.
"""

helps['functionapp deployment source config'] = """
    type: command
    short-summary: Associate to Git or Mercurial repositories.
"""

helps['functionapp deployment source config-local-git'] = """
    type: command
    short-summary: Enable local git.
    long-summary: Get an endpoint to clone and later push to the function app.
    examples:
        - name: Get a git endpoint for a web app and add it as a remote.
          text: >
            az functionapp source-control config-local-git \\
                -g MyResourceGroup -n MyUniqueApp

            git remote add azure \\
                https://<deploy_user_name>@MyUniqueApp.scm.azurewebsites.net/MyUniqueApp.git
"""

helps['functionapp deployment source delete'] = """
    type: command
    short-summary: Delete source control configurations.
"""

helps['functionapp deployment source show'] = """
    type: command
    short-summary: Show source control configurations.
"""

helps['functionapp deployment source sync'] = """
    type: command
    short-summary: Synchronize from the source repository, only needed under manual integration mode.
"""
helps['functionapp deployment user'] = """
    type: group
    short-summary: Manage user credentials for a deployment.
"""

helps['functionapp deployment user set'] = """
    type: command
    short-summary: Update deployment credentials.
    long-summary: All function/web apps in the subscription will be impacted since all apps share
                  the same deployment credentials.
    examples:
        - name: Set FTP and git deployment credentials for all function/web apps.
          text: >
            az functionapp deployment user set
            --user-name MyUserName
"""
